import requests
import json
import argparse
import jwt
import uuid
import time
from OpenSSL import crypto
from constants import * 
from requests.packages.urllib3.exceptions import InsecureRequestWarning
# Disable the InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

APP_REDIRECT_URI = f"https://{FASPEX_HOSTNAME}/aspera/faspex/"
FASPEX_AUTH_TOKEN_URL = f"{APP_REDIRECT_URI}/auth/token"

def authenticate_and_get_token(debug=False):

    time_since_epoch = int(time.time())

    payload = {
        "iss": FASPEX_CLIENT_ID,
        "sub": f"user:{email}",
        "aud": FASPEX_CLIENT_ID,
        "jti": str(uuid.uuid4()),
        "exp": time_since_epoch + 103600,
        "iat": time_since_epoch
    }

    if debug:
        print("JWT Payload:")
        print(payload)

    with open(private_key_file) as key_file:
        private_key = key_file.read()

    token = jwt.encode(payload=payload, key=private_key, algorithm="RS256")

    body = {
        "client_id": FASPEX_CLIENT_ID,
        "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
        "redirect_uri": APP_REDIRECT_URI,
        "state": str(uuid.uuid4()),
        "assertion": token
    }

    with open(public_key_file) as pub_key_file:
        pub_key = pub_key_file.read()

    decoded_token = jwt.decode(token, pub_key, verify=True, algorithms="RS256", audience=FASPEX_CLIENT_ID)

    if debug:
        print("Encoded token:")
        print(token)
        print("Decoded token:")
        print(decoded_token)

    response = requests.post(FASPEX_AUTH_TOKEN_URL, json=body, verify=False, allow_redirects=True)

    if debug:
        print("Authentication Response:")
        print("Status Code:", response.status_code)
        print("Reason:", response.reason)
        print("Response Text:")
        print(response.text)

    token = json.loads(response.text)['access_token']

    print("\nBearer Token: ")
    print(token)

    return token

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Authenticate and get token')
    parser.add_argument('--debug', action='store_true', help='Enable debug mode')

    args = parser.parse_args()
    authenticate_and_get_token(debug=args.debug)